/* TestCloneFromImageFile.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo53/src/packages/ibmee/javaapi/examples/TestCloneFromImageFile.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBProfileCreationConfiguration;
import com.ibm.rbapi.RBProfileCreationParams;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBWebExtensionStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

/*
 * Example class to show capturing a cloned image from a reference image file.
 * 
 * The reference image file can be a Windows Vista WIM image, a Solaris Flash
 * archive, or a Rembo Toolkit header.  Note that for Rembo Toolkit images,
 * you should look at the example class "TestToolkitProfileCreation" instead.
 * 
 * This is similar to what is done in the TestRBImageCaptureTemplate example,
 * however this is the preferred method. It enables you to probe the host first
 * and determine some information about it before creating the system profile.
 */
public class TestCloneFromImageFile {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestCloneFromImageFile <ip> <port> <password> \n");
            return;
        }

        System.out.println("TestCloneFromImageFile starting. We will test capturing a clone from a reference image file.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        boolean isWimImage = true;
        while (true) {
            System.out.println("\nChoose which type of image file you wish to work with: ");
            System.out.println("0) Windows Vista wim image");
            System.out.println("1) Solaris Flash archive");
            try {
                int imgFileChoice = Integer.parseInt(inReader.readLine().trim());
                if (imgFileChoice >= 0 && imgFileChoice <= 1) {
                    if (imgFileChoice == 1) {
                        isWimImage = false;
                    }
                    break;
                }
            } catch (NumberFormatException nfe) {}
        }
        
        
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test class needs at least one host running the WIE to run properly.");
                System.out.println("Please start a WIE connecting to the TPMfOSd server." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host running the WIE on which you wish to perform the clone capture from the reference image file: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbWIE = hostRecs.elementAt(hostIndex);
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbWIE.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbWIE.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
       
        String installFolder = "";
        if (isWimImage) {
            System.out.println("Enter the full path to the wim image file (this should be a path on the host running the WIE):");
            installFolder = inReader.readLine().trim();
        } else {
            System.out.println("\nEnter the directory on the RBHost running the WIE in which you have\n" +
                    "prepared a WANboot miniroot image for starting the installation process.\n" +
                    "A miniroot image is typically prepared by running the command setup_install_server\n" +
                    "with argument -w, using your Solaris installation DVD.\n" +
                    "Make sure that the Solaris version in the miniroot image you specify matches the\n" +
                    "version of the operating system you want to install: \n");
            String miniroot = inReader.readLine().trim();
            
            System.out.println("\nEnter the full path (on the RBHost running the WIE) to the flash archive to deploy.\n" +
                    "The flash archive must have been created using the Solaris 'flar' utility:\n"); 
            String flashimage = inReader.readLine().trim();
            
            // Install folder for solaris cloned from img file should be: "flashimage;minirootdir"
            installFolder = flashimage + ";" + miniroot;
        }
        
        
        System.out.println("\nWe will now run a probe of the image file on the RBHost running the WIE...\n"); 
        RBProfileCreationParams profCreateParams = null;
        boolean successful = false;
        int count = 0;
        int timeout = 60;
        while (count <= 3) {
            try {
                count++; 
                System.out.println("Probing the with timeout of: " + timeout + " seconds."); 
                profCreateParams = rbServer.probeImageFile(rbWIE, installFolder, timeout);
                if(profCreateParams.getStatus().equals("COMPLETED")) {
                    System.out.println("Probe completed successfully.");
                    successful = true;
                } else {
                    System.out.println("Unexpected probe status - Status returned is: " + profCreateParams.getStatus());
                    printMap(profCreateParams.getError());
                }
                break;
            } catch (RBTimeoutException tme) {
                System.out.println("Probe timed out.");
                timeout = timeout + 20;
            }
        }
        if (!successful) {
            System.out.println("Probe did NOT complete successfully.");
            System.exit(2);
        }
        
        // Let's print out some information that was gathered during the probe.
        System.out.println("ProbedOS- description: " + profCreateParams.getDescription());
        System.out.println("ProbedOS- proftype: " + profCreateParams.getProfileType());
        System.out.println("ProbedOS- ostype: " + profCreateParams.getOSType());
        RBProfileCreationConfiguration[] configs = profCreateParams.getConfigurations();
        for (int i=0; i<configs.length; i++) {
            System.out.println("ProbedOS- configs["+i+"] - desc: " + configs[i].getDescription());
            System.out.println("ProbedOS- configs["+i+"] - createconfig?: " + configs[i].createConfiguration());
            System.out.println("ProbedOS- configs["+i+"] - syspreped: " + configs[i].getSyspreped());
        }
        
        System.out.println("We will capture the clone of the host from the image file, creating a system profile\n"
                + "with the description: " + profCreateParams.getDescription() + ".");
        System.out.println("Please enter a comment for the system profile: ");
        String comment = inReader.readLine().trim();
        // Could also change the description here if you want
        //profCreateParams.setDescription("my description");
        profCreateParams.setComment(comment);
        
        Date now = new Date();
        Date later = new Date(now.getTime()+(1000*3600*24));
        RBActivity imgCapAct = rbServer.registerCloneFromImageProfileCreation("create clone from img file test", 
                true, null, profCreateParams, rbWIE, now, later, later);
        System.out.println("\nCreated new activity to create the clone immediately ");
        
        checkForActivityFinished(imgCapAct, now);

        System.out.println("Clone from img file completed...");
        System.out.println("Done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    // polling to see if our profile creation activity completes. This is NOT 
    // very efficient for polling activity status, and is just here to make this
    // example simple. For a better example of how to monitor activity status, see TestStressSmart.java
    // This example also assumes there is only one activity target for the activity.
    private static RBActivityTargetResults checkForActivityFinished(RBActivity act, Date startTime) 
    throws RBServerException, RBObjectNotFoundException, InterruptedException 
    {
        RBActivityTargetResults actTargetResults = null;
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        /* Now print out some status for the activity running on our Target */
        int count = 0;  // Loop through and exit when we find the first finished event
        Date lastTime = startTime;
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent && count < 150 ) {
            Thread.sleep(10000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events = act.getEventsFromServer(lastTime);

            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + " Status=" + e.getStatus());

                // Let's get information about events that are completed or failed
                if (e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Event status is: " + e.getStatus() + " Now let's get some more info from the server\n"); 

                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget rbActTarget = e.getRBActivityTarget();
                    System.out.println("Getting RBActivityTargetResults object");
                    actTargetResults = rbActTarget.getActivityTargetResults();
                    System.out.println("Results: errors -- ");
                    printMap(actTargetResults.getError());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
            count++;
        }
        
        if (actTargetResults == null) {
            System.out.println("Did not find results of activity (perhaps we stopped checking too soon?).\n" +
                    "Exiting.");
            System.exit(10);
        }
        if (!actTargetResults.getStatus().equals("COMPLETED")) {
            System.out.println("System profile creation activity did not complete successfully.\n" +
                    "Exiting.");
            System.exit(11);
        }
        return actTargetResults;
    }
}
