/* TestCloneHost.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo53/src/packages/ibmee/javaapi/examples/TestCloneHost.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBProfileCreationConfiguration;
import com.ibm.rbapi.RBProfileCreationParams;
import com.ibm.rbapi.RBProfileCreationTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

/*
 * Example class to show capturing a cloned image from a host (Creating a new profile
 * by capturing a clone from a reference machine).
 * 
 * This is similar to what is done in the TestRBImageCaptureTemplate example,
 * however this is the preferred method. It enables you to probe the host first
 * and determine some information about it before creating the system profile.
 */
public class TestCloneHost {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestCloneHost <ip> <port> <password> \n");
            return;
        }

        System.out.println("TestCloneHost starting. We will test capturing a clone from a host machine");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test class needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to capture the clone of: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = hostRecs.elementAt(hostIndex);
        
        System.out.println("\nRegistering an activity to probe the host OS. Please ensure your \n" +
        		"host is on and network booted from the TPMfOSd server.");
        Date now = new Date(); 
        Date later = new Date(now.getTime() + (1000*3600));
        RBActivity probeAct = rbServer.registerProbeHostOSActivity("Test probe host OS", 
                true, false, null, new RBHost[] {rbhost}, now, later, later);
        
        // Wait for the activity to complete and get the ActivityTargetResults.
        RBActivityTargetResults probeResults = checkForActivityFinished(probeAct, now);
        
        // Get the Profile creation params from the activity target results.
        RBProfileCreationParams profCreateParams = probeResults.getProfileCreationParameters();
        
        // Let's print out some information that was gathered during the probe.
        System.out.println("ProbedOS- description: " + profCreateParams.getDescription());
        System.out.println("ProbedOS- proftype: " + profCreateParams.getProfileType());
        System.out.println("ProbedOS- ostype: " + profCreateParams.getOSType());
        System.out.println("ProbedOS- diskcount: " + profCreateParams.getDiskCount());
        System.out.println("ProbedOS- capturealldisks?: " + profCreateParams.isCaptureAllDisks());
        System.out.println("ProbedOS- repairBCD?: " + profCreateParams.isRepairBCD());
        RBProfileCreationConfiguration[] configs = profCreateParams.getConfigurations();
        for (int i=0; i<configs.length; i++) {
            System.out.println("ProbedOS- configs["+i+"] - desc: " + configs[i].getDescription());
            System.out.println("ProbedOS- configs["+i+"] - createconfig?: " + configs[i].createConfiguration());
            System.out.println("ProbedOS- configs["+i+"] - syspreped: " + configs[i].getSyspreped());
        }
        
        RBProfileCreationTemplate myCloneProfTemplate = rbServer.registerProfileCreationTemplate(
                "Test-CaptureClone", RBProfileCreationTemplate.PROFTYPE_CLONING, false, false, false, false);
        
        System.out.println("We will capture the clone of the host, creating a system profile\n"
                + "with the description: " + profCreateParams.getDescription() + ".");
        System.out.println("Please enter a comment for the system profile: ");
        String comment = inReader.readLine().trim();
        // Could also change the description here if you want
        //profCreateParams.setDescription("my description");
        profCreateParams.setComment(comment);
        // Default behavior is to capture just the 1st disk - uncomment the 
        // line below to capture all disks.
        //profCreateParams.setCaptureAllDisks(true);
        
        Date start = new Date();
        HashMap paramMap = new HashMap();
        paramMap.put("RBProfileCreationParams", profCreateParams);     
        RBActivity imgCapAct = rbServer.registerActivity("capture clone test", 
                myCloneProfTemplate, true, false, paramMap, rbhost);
        System.out.println("\nCreated new activity to capture the clone immediately ");
        
        checkForActivityFinished(imgCapAct, start);

        System.out.println("Clone capture completed...");
        // Let's remove our activity template to clean up. Note that normally
        // you may re-use templates rather than creating/deleting one every time.
        rbServer.removeTemplate(myCloneProfTemplate, true);
        System.out.println("Done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    // polling to see if our profile creation activity completes. This is not 
    // very efficient for polling activity status, and is just here to make this
    // example simple. For a better example of how to monitor activity status, see TestStressSmart.java
    // This example also assumes there is only one activity target for the activity.
    private static RBActivityTargetResults checkForActivityFinished(RBActivity act, Date startTime) 
    throws RBServerException, RBObjectNotFoundException, InterruptedException 
    {
        RBActivityTargetResults actTargetResults = null;
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        /* Now print out some status for the activity running on our Target */
        int count = 0;  // Loop through and exit when we find the first finished event
        Date lastTime = startTime;
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent && count < 150 ) {
            Thread.sleep(10000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events = act.getEventsFromServer(lastTime);

            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + " Status=" + e.getStatus());

                // Let's get information about events that are completed or failed
                if (e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Event status is: " + e.getStatus() + " Now let's get some more info from the server\n"); 

                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget rbActTarget = e.getRBActivityTarget();
                    System.out.println("Getting RBActivityTargetResults object");
                    actTargetResults = rbActTarget.getActivityTargetResults();
                    System.out.println("Results: errors -- ");
                    printMap(actTargetResults.getError());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
            count++;
        }
        
        if (actTargetResults == null) {
            System.out.println("Did not find results of activity (perhaps we stopped checking too soon?).\n" +
                    "Exiting.");
            System.exit(10);
        }
        if (!actTargetResults.getStatus().equals("COMPLETED")) {
            System.out.println("System profile creation activity did not complete successfully.\n" +
                    "Exiting.");
            System.exit(11);
        }
        return actTargetResults;
    }
}
