/* TestSoftwarePackageCreation.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestSoftwarePackageCreation.java#2 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBDriverContentInfo;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackageParams;
import com.ibm.rbapi.RBWebExtensionStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestSoftwarePackageCreation {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        if (args.length < 3) {
            System.out.println("usage: TestSoftwarePackageCreation <ip> <port> <password>");
            return;
        }
        
        System.out.println("TestSoftwarePackageCreation starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        System.out.println("Created rbServer.\n");
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to use: \n");
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.  " +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        boolean isWinDriver = false;
        
        int pkgType = 0;
        while (pkgType < 1 || pkgType > 10) {
            System.out.println("\n\nPlease enter the number of the Software Package type you \n" +
            "wish to create:");
            System.out.println("1) MSI package");
            System.out.println("2) Floppy disk image");
            System.out.println("3) Win PE image");
            System.out.println("4) Win PE2 image");
            System.out.println("5) RPM package");
            System.out.println("6) fileset creation package");
            System.out.println("7) Linux preboot image");
            System.out.println("8) Windows Vista hotfix (MSU) package");
            System.out.println("9) Windows Vista language pack");
            System.out.println("10) Windows driver");
            
            // NOTE, there are more types - using these 10 as an example
            String packageType = inReader.readLine();
            pkgType = Integer.parseInt(packageType);
        }
        
        System.out.println("\n\nPlease enter the path to the directory containing your \n" +
           "file(s) that will be packaged.  This should be the path on the host.");
        String sourcePath = inReader.readLine();
        
        RBSoftwarePackageParams softPkgParams = null;
        switch (pkgType) {
            case 1:  
                System.out.println("Creating MSI package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newMSICreationParams("test MSI pkg", 
                          "msi comment", sourcePath, 
                          "c:\\installTest\\testmsi", "");
                break;
            case 2:  
                System.out.println("Creating Floppy package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newFloppyCreationParams("testFloppyPkg", 
                          "floppy comment", sourcePath);
                break;
            case 3:  
                System.out.println("Creating Win PE package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newWinPECreationParams("testWinPEPkg", 
                            "winpe comment", sourcePath);
                break;
            case 4:
                System.out.println("Creating Win PE2 package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newWinPE2CreationParams("testWinPE2Pkg", 
                            "winpe2 comment", sourcePath);
                break;
            case 5:
                System.out.println("Creating RPM package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newRPMCreationParams("testRPMpkg", 
                            "rpm comment", sourcePath, "", "rpm -ivh");
                break;
            case 6:
                System.out.println("Creating new fileset creation package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newFileSetCreationParams("testFileSetPkg", 
                            "fileset comment", sourcePath, "", "cmd", 
                            false, false);
                break;
            case 7:
                System.out.println("Creating new Linux preboot package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newLinuxPECreationParams("testLinuxPEPkg", 
                            "Linux kernel and initial ramdisk", sourcePath, "bootimg", "ramdisk");
                break;
            case 8:
                System.out.println("Creating new Windows HotFix (MSU) package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newWinHotFixCreationParams("testWinHotFixPkg", 
                            "Test Windows hotfix package", sourcePath, null, true);
                break;
            case 9:
                System.out.println("Creating new Windows Language pack params");
                softPkgParams = 
                    RBSoftwarePackageParams.newWinLangCreationParams("testWinLangPkg", 
                            "Test Windows language package", sourcePath, null);
                break;
            case 10:
                System.out.println("Creating new Windows driver params");
                isWinDriver = true;
                softPkgParams = 
                    RBSoftwarePackageParams.newWinDriverCreationParams("AAAtestWinDrvrPkg", 
                            "Test Windows driver package", sourcePath, null, true);
                break;
            default: 
                System.out.println("Invalid selection.  Exiting.");
                System.exit(-1);
        }
        
        /*
         * We can decide if we want to remove the installation files after
         * installing the software package.
         */
        System.out.println("Do you wish to remove the install files after " +
                "installation of this SoftwarePackage? (This is not valid" +
                "for soft pkgs such as dos preboot packages)\n" +
                "(Enter y for yes, default is no): [n] ");
        String removeFilesString = inReader.readLine().trim().toLowerCase();
        boolean removeFiles = false;
        if (removeFilesString.equals("y") || removeFilesString.equals("yes")) {
            removeFiles = true;
        }
        /* 
         * Now let's set in the software package params if we want to remove
         * files after install or not. (note that this doesn't really do 
         * anything for soft pkgs that don't install files - for example
         * dos preboot pkgs).
         */
        softPkgParams.setRemoveFiles(removeFiles);
        
        
        System.out.println("Create softwarePackageParams done.");
        printSoftwarePackageParams(softPkgParams, false);
        
        System.out.println("\n\nPerforming a probeSoftwarePackageCreation on the target...");
        RBActivityTargetResults probeResults = null;
        for (int i=0; i<2; i++) {
            System.out.println("Probe attempt #" + i);
            try {
                probeResults = rbServer.probeSoftwarePackageCreation(rbhost, softPkgParams, 30+(i*10));
                break;
            } catch (RBTimeoutException timeEx) {
                timeEx.printStackTrace();
                System.out.println("Probe timed out.");
            }
        }
        if (probeResults == null) {
            System.out.println("Probe is not completing within the timeout.  Exiting.");
            System.exit(1);
        }
        System.out.println("\nProbe done.");
        
        System.out.println("status of probe: " + probeResults.getStatus());
        if (!probeResults.getStatus().equals(RBActivityTarget.COMPLETED)) {
            System.out.println("Probe did not complete successfully.  Printing out the error:");
            printMap(probeResults.getError());
            System.out.println("\n\nExiting.");
            System.exit(-1);
        }
            
        System.out.println("Printing out updated SoftwarePackageParams from probe: ");
        printSoftwarePackageParams(softPkgParams, isWinDriver);
        
        /*
         * Let's set some extra parameters if this is a windows driver
         */
        RBSoftwarePackageParams[] driverPkgParams = null;
        if (isWinDriver) {
            if (softPkgParams.getType().equals("multi")) {
                // This means the probe has found multiple drivers - Let's give
                // the user a choice whether they want to create software packages
                // for them all or just a subset.
                System.out.println("Which drivers do you want to create software packages for? Default will be all drivers");
                boolean x86DriversFound = false;
                boolean x8664DriversFound = false;
                RBDriverContentInfo[] driverInfo = softPkgParams.getDriverContentInfo();
                for (int i=0; i<driverInfo.length; i++) {
                    System.out.println(i + ") driver"+i+": ");
                    System.out.println("    SubPath: " + driverInfo[i].getSubPath());
                    System.out.println("    Provider: " + driverInfo[i].getProvider());
                    System.out.println("	Class: " + driverInfo[i].getDriverClass());
                    System.out.println("	Version: " + driverInfo[i].getDriverVersion());
                    System.out.println("	Architecture: " + driverInfo[i].getOSArchitecture());
                    if (driverInfo[i].getOSArchitecture().equals("x86")) {
                        x86DriversFound = true;
                    } else if (driverInfo[i].getOSArchitecture().equals("x86-64")) {
                        x8664DriversFound = true;
                    }
                    System.out.println("    Total Devs: " + driverInfo[i].getTotalDevices());
                    System.out.println("    Total Rebranded Variants: " + driverInfo[i].getTotalVariants());
                    System.out.println("    Devices: ");
                    String[] devs = driverInfo[i].getDevices();
                    for (int j=0; j<devs.length; j++) {
                        System.out.println("        - " + devs[j]);
                    }
                }
                int optionsCount = driverInfo.length;
                if (x86DriversFound) {
                    System.out.println(optionsCount+") All x86 drivers");
                    optionsCount++;
                }
                if (x8664DriversFound) {
                    System.out.println(optionsCount+") All x86-64 drivers");
                    optionsCount++;
                }
                System.out.println(optionsCount+") All drivers (default)");
                
                int dChoice = optionsCount;
                try {
                    dChoice = Integer.parseInt(inReader.readLine().trim());
                } catch (NumberFormatException e) {
                }
                
                if (dChoice >= 0 && dChoice < driverInfo.length) {
                    System.out.println("\nCreating software package for driver" +dChoice+ "...");
                   // subPaths.add(driverInfo[dChoice].getSubPath());
                    
                    // Since we did a probe, type was changed to "multi" which means
                    // that the software package creation will create pkgs for all drivers.
                    // We have 2 choices:  1 - change type back to "pkg" and specify the
                    // subpath we want, or 2 - simply create a new RBSoftwarePackageParams
                    // by calling newWinDriverCreationParams()(type will automatically be set 
                    // to "pkg" and set the subpath we want on this new RBSoftwarePackageParams.
                    // We will use method 1 here, and method 2 for the x86-64 or x86 choices below.
                    softPkgParams.setPkgDescr(driverInfo[dChoice].getProvider()+"-driver"+dChoice);
                    softPkgParams.setType("pkg");
                    softPkgParams.setSubPath(driverInfo[dChoice].getSubPath());
                    driverPkgParams = new RBSoftwarePackageParams[] {softPkgParams};
                } else if (x8664DriversFound && dChoice == (optionsCount-1)) {
                    System.out.println("\nCreating software packages only for x86-64 drivers...");
                    ArrayList myDrivers = new ArrayList();
                    for (int i=0; i<driverInfo.length; i++) {
                        if (driverInfo[i].getOSArchitecture().equals("x86-64")) {
                            RBSoftwarePackageParams newDriverParams = RBSoftwarePackageParams.newWinDriverCreationParams(
                                    driverInfo[i].getProvider()+"x86-64-driver"+i, "drvr pkg created from example program", 
                                    sourcePath, null, true);
                            // Set the appropriate subpath
                            newDriverParams.setSubPath(driverInfo[i].getSubPath());
                            myDrivers.add(newDriverParams);
                        }
                    }
                    driverPkgParams = (RBSoftwarePackageParams[]) myDrivers.toArray(new RBSoftwarePackageParams[0]);
                } else if ((x8664DriversFound && dChoice == (optionsCount-2)) 
                        || (!x8664DriversFound && dChoice == (optionsCount-1))) {
                    System.out.println("\nCreating software packages only for x86 drivers...");
                    ArrayList myDrivers = new ArrayList();
                    for (int i=0; i<driverInfo.length; i++) {
                        if (driverInfo[i].getOSArchitecture().equals("x86")) {
                            RBSoftwarePackageParams newDriverParams = RBSoftwarePackageParams.newWinDriverCreationParams(
                                    driverInfo[i].getProvider()+"x86-driver"+i, "drvr pkg created from example program", 
                                    sourcePath, null, true);
                            // Set the appropriate subpath
                            newDriverParams.setSubPath(driverInfo[i].getSubPath());
                            myDrivers.add(newDriverParams);
                        }
                    }
                    driverPkgParams = (RBSoftwarePackageParams[]) myDrivers.toArray(new RBSoftwarePackageParams[0]);
                    for (int i=0; i<driverPkgParams.length; i++) {
                        System.out.println("drverpkgparams--subpath: " + driverPkgParams[i].getSubPath());
                    }
                } else { 
                    System.out.println("\nCreating software packages for all drivers...");
                    // Multiple driver packages will be created.  This since the type
                    // is set to "multi", all drivers in the source path will have
                    // software packages created for them with one activity.
                    // We should give a Prefix - all the descriptions of the newly
                    // created drivers will have this prefix so they can be easily found.
                    System.out.println("\nThere are multiple drivers found - when we " +
                    		"create the drivers, we will prepend a prefix to the description\n" +
                    		"For each driver created.  Please enter a one-word prefix (no spaces): \n");
                    String multiDriverPrefix = inReader.readLine().trim();
                    softPkgParams.setMultiDriverPrefix(multiDriverPrefix);
                    System.out.println("Newly created drivers will all have a description\n" +
                    		"that starts with: " + multiDriverPrefix);
                    softPkgParams.setPkgDescr("Test-AllDrivers");
                    driverPkgParams = new RBSoftwarePackageParams[] {softPkgParams};
                }
                
                
            } else {
                driverPkgParams = new RBSoftwarePackageParams[] {softPkgParams};
            }
            
            /*
             * Options for binding rules here - to make it easy in this example, 
             * even if we are creating multiple drivers, let's use the same
             * automatic binding rule creation params for all of them.
             */
            // We already chose "true" to creating binding rules when calling
            // RBSoftwarePackageParams.newWinDriverCreationParams() earlier,
            // but we can still change our mind...
            System.out.println("\nYou can create binding rules describing when your driver must \n" +
                    "be deployed with the operating system. Therefore this driver will be \n" +
                    "automatically deployed without any human intervention. These binding rules \n" +
                    "can be based on several host characteristics.\n");
            System.out.println("Do you wish to create binding rules for this windows driver? ");
            System.out.println("(Enter y for yes, anything else for no, default is yes): [y] ");
            String createRules = inReader.readLine().trim().toLowerCase();
            if (createRules.equals("y") || createRules.equals("yes") || createRules.equals("")) {
                // OK, let's have some binding rules created when we create the software package...
                System.out.println("\nCreate binding rules based on PCI Hardware ID?");
                System.out.println("(Enter y for yes, anything else for no, default is yes): [y] ");
                String answer = inReader.readLine().trim().toLowerCase();
                if (answer.equals("y") || answer.equals("yes") || answer.equals("")) {
                    for (int i=0; i<driverPkgParams.length; i++) {
                        driverPkgParams[i].setBindToPCIHardwareID(true);
                    }
                    
                    System.out.println("\nBind to the exact same device only? No means we will bind to similar devices.");
                    System.out.println("(Enter y for yes, anything else for no, default is no): [n] ");
                    answer = inReader.readLine().trim().toLowerCase();
                    if (answer.equals("y") || answer.equals("yes")) {
                        for (int i=0; i<driverPkgParams.length; i++) {
                            driverPkgParams[i].setBindToPCIExactDeviceOnly(true);
                        }
                    }
                } else {
                    for (int i=0; i<driverPkgParams.length; i++) {
                        driverPkgParams[i].setBindToPCIHardwareID(false);
                    }
                }
                
                System.out.println("\nCreate binding rules based on Baseboard ID?");
                System.out.println("(Enter y for yes, anything else for no, default is no): [n] ");
                answer = inReader.readLine().trim().toLowerCase();
                if (answer.equals("y") || answer.equals("yes")) {
                    System.out.println("\nEnter the target machine baseboard ID that " +
                            "should be matched to apply this driver: ");
                    answer = inReader.readLine().trim();
                    for (int i=0; i<driverPkgParams.length; i++) {
                        driverPkgParams[i].setBindToBaseBoard(answer);
                    }
                }
                
                System.out.println("\nCreate binding rules based on Model?");
                System.out.println("(Enter y for yes, anything else for no, default is no): [n] ");
                answer = inReader.readLine().trim().toLowerCase();
                if (answer.equals("y") || answer.equals("yes")) {
                    System.out.println("\nEnter the target machine model that should " +
                            "be matched to apply this driver: ");
                    answer = inReader.readLine().trim();
                    for (int i=0; i<driverPkgParams.length; i++) {
                        driverPkgParams[i].setBindToModel(answer);
                    }
                }
                
                int answerInt = -1;
                System.out.println("\n\n1) auto");
                System.out.println("2) 32-bit (x86)");
                System.out.println("3) 64-bit (x86-64)");
                System.out.println("4) any");
                System.out.println("Enter the OS Architecture to be used, or enter to continue with the default. \n" +
                        "Default is: auto");
                answer = inReader.readLine().trim();
                try {
                    answerInt = Integer.parseInt(answer);
                } catch (NumberFormatException e) {
                    answerInt = 0;
                }
                String bindToOSArchString = null;
                switch (answerInt) {
                case 1:  
                    bindToOSArchString = RBSoftwarePackageParams.BINDRULE_OSARCH_AUTO;
                    break;
                case 2:
                    bindToOSArchString = RBSoftwarePackageParams.BINDRULE_OSARCH_32BIT;
                    break;
                case 3:
                    bindToOSArchString = RBSoftwarePackageParams.BINDRULE_OSARCH_64BIT;
                    break;
                case 4:
                    bindToOSArchString = RBSoftwarePackageParams.BINDRULE_OSARCH_ANY;
                    break;
                default:
                    bindToOSArchString = RBSoftwarePackageParams.BINDRULE_OSARCH_AUTO;
                    break;
                }
                for (int i=0; i<driverPkgParams.length; i++) {
                    driverPkgParams[i].setBindToOSArchitecture(bindToOSArchString);
                }
                
                
                String bind2OSType = "Windows XP%;Windows 2003%";
                answerInt = 0;
                do {
                    System.out.println("\n\n1) Windows 2000");
                    System.out.println("2) Windows XP");
                    System.out.println("3) Windows Server 2003");
                    System.out.println("4) Windows Vista/Longhorn");
                    System.out.println("5) Clear current selection");
                    System.out.println("Enter a target OS to be bound to, or enter to continue with current selection. \n" +
                            "Current selection is: " + bind2OSType);
                    answer = inReader.readLine().trim();
                    try {
                        answerInt = Integer.parseInt(answer);
                    } catch (NumberFormatException e) {
                        answerInt = 0;
                    }
                    switch (answerInt) {
                    case 1:
                        bind2OSType = (bind2OSType.equals("") ? "" : bind2OSType+";") + "Windows 2000%";
                        break;
                    case 2:
                        bind2OSType = (bind2OSType.equals("") ? "" : bind2OSType+";") + "Windows XP%";
                        break;
                    case 3:
                        bind2OSType = (bind2OSType.equals("") ? "" : bind2OSType+";") + "Windows 2003%";
                        break;
                    case 4:
                        bind2OSType = (bind2OSType.equals("") ? "" : bind2OSType+";") + "Windows Vista%";
                        break;
                    case 5:
                        bind2OSType = "";
                        break;
                    default:
                        answerInt = 0;
                        break;
                    }
                } while (answerInt != 0);
                
                // Now bind to the OSType selected above.
                for (int i=0; i<driverPkgParams.length; i++) {
                    driverPkgParams[i].setBindToOSType(bind2OSType);
                }
                
                        
                System.out.println("\n\nFor Windows driver creation - Binding rules to be created:");
                System.out.println("Bind to PCI Hardware ID: " + driverPkgParams[0].isBindToPCIHardwareID());
                if (driverPkgParams[0].isBindToPCIHardwareID()) {
                    System.out.println("Bind to exact PCI device only: " + driverPkgParams[0].isBindToPCIExactDeviceOnly());
                }
                System.out.println("BindToBaseboard: " + (driverPkgParams[0].getBindToBaseBoard()==null?"No rule":driverPkgParams[0].getBindToBaseBoard()));
                System.out.println("BindToModel: " + (driverPkgParams[0].getBindToModel()==null?"No rule":driverPkgParams[0].getBindToModel()));
                System.out.println("BindToOSArchitecture: " + (driverPkgParams[0].getBindToOSArchitecture()==null?"No rule":driverPkgParams[0].getBindToOSArchitecture()));
                System.out.println("BindToOSType: " + (driverPkgParams[0].getBindToOSType()==null?"No rule":driverPkgParams[0].getBindToOSType()));
            } else {
                // Do not create binding rules at all
                for (int i=0; i<driverPkgParams.length; i++) {
                    driverPkgParams[i].setCreateRules(false);
                }
            }
        } 
        
        
        // Create the software package
        if (isWinDriver) {
            for (int i=0; i<driverPkgParams.length; i++) {
                System.out.println("Creating driver software package: " + driverPkgParams[i].getPkgDescr());
                createSoftPkg(driverPkgParams[i], rbServer, rbhost);
            }
        } else {
            System.out.println("Creating software package: " + softPkgParams.getPkgDescr());
            createSoftPkg(softPkgParams, rbServer, rbhost);
        }
        
        System.out.print("\nTestSoftwarePackageCreation done.");
    }
    
    
    private static void createSoftPkg(RBSoftwarePackageParams spParams, RBServer serverToUse, RBHost agentToUse) 
    throws RBServerException, RBObjectNotFoundException, InterruptedException {
        Date now = new Date();
        Date stop = new Date(now.getTime()+1000*3600);
        Date expire = new Date(now.getTime()+1000*3600*24);
        String softPkgName = new String(spParams.getPkgDescr());
        RBActivity act = serverToUse.registerSoftwarePackageCreationActivity("TestImageCapture", 
                true, null, agentToUse, spParams, now, stop, expire);
        
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent) {
            Thread.sleep(8000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events =  act.getEventsFromServer(lastTime);
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];
    
                System.out.println("Event: Progress=" + e.getProgress() + 
                        " Progress Msg=" + e.getLastProgressMsg() + 
                        " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget ourActTarget = e.getRBActivityTarget();
                    System.out.println("Activity Target: Progress=" + ourActTarget.getProgress() + 
                            " Progress Msg=" + ourActTarget.getLastProgressMsg() + 
                            " Status=" + ourActTarget.getStatus());
                    
                    foundCompletedEvent = true;
                    System.out.println("Exiting loop. - Finished creating software package: " + softPkgName);
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
    }
    
    
    private static void printMap(Map map) {
        System.out.println("Printing Error from result: " );
        Set keys = map.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            String key = (String) keyIt.next();
            System.out.println("    key: " + key
                    + " -> " + (String) map.get(key));
        }
    }
    
    private static void printSoftwarePackageParams(RBSoftwarePackageParams softPkgParams, boolean isWinDriver) {
        System.out.println("\n### Printing out software package Params ###");
        System.out.println("Package Description: " + softPkgParams.getPkgDescr());
        System.out.println("Package Comment: " + softPkgParams.getComment());
        System.out.println("Destination Path: " + softPkgParams.getDestPath());
        System.out.println("MSI file name: " + softPkgParams.getMsiFileName());
        System.out.println("MSI file names list: " + softPkgParams.getMsiFileNamesList());
        System.out.println("Package file name: " + softPkgParams.getPackageFileName());
        System.out.println("Product name: " + softPkgParams.getProductName());
        System.out.println("Product version: " + softPkgParams.getProductVersion());
        System.out.println("Source path: " + softPkgParams.getSourcePath());
        System.out.println("Vendor: " + softPkgParams.getVendor());
        System.out.println("Remove Files after install: " + softPkgParams.isRemoveFiles());
        
//        if (isWinDriver) {
//            System.out.println("Type is: " + softPkgParams.getType());
//            System.out.println("\nPrinting some extra parameters specific to Windows Driver type...");
//            RBDriverContentInfo[] driverInfo = softPkgParams.getDriverContentInfo();
//            for (int i=0; i<driverInfo.length; i++) {
//                System.out.println("DriverInfo["+i+"]:");
//                System.out.println(" - SubPath: " + driverInfo[i].getSubPath());
//                System.out.println(" - Provider: " + driverInfo[i].getProvider());
//                System.out.println(" - Class: " + driverInfo[i].getDriverClass());
//                System.out.println(" - Version: " + driverInfo[i].getDriverVersion());
//                System.out.println(" - Architecture: " + driverInfo[i].getOSArchitecture());
//                System.out.println(" - Total Devs: " + driverInfo[i].getTotalDevices());
//                System.out.println(" - Total Rebranded Variants: " + driverInfo[i].getTotalVariants());
//                System.out.println(" - Devices: ");
//                String[] devs = driverInfo[i].getDevices();
//                for (int j=0; j<devs.length; j++) {
//                    System.out.println("  -- " + devs[j]);
//                }
//            }
//        }
    }
}
