/* TestRBServerStatus.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/osd511/src/packages/ibmee/javaapi/examples/TestSubnets.java#1 $
 */
package examples;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;

import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSubnet;
import com.ibm.rbapi.RBSubnetRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestSubnets {

    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, IOException, RBTimeoutException, RBObjectNotFoundException
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestSubnets <ip> <port> <password>");
            return;
        }

        System.out.println("TestSubnets starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret, Level.ALL); 
        System.out.println("Created rbServer.\n");
        
        System.out.println("RBServer.isReady() returns: " + rbServer.isReady(15000));
        
        RBSubnetRecs recs = rbServer.getSubnets("","");     // usual filter and order
        RBSubnet[] subnets = recs.subList(0, recs.size());
        RBSubnet current = null;
        for (int i=0; i<subnets.length; i++) {
            current = subnets[i];
            System.out.println("ip:"+current.getIPAddress()
                    +",mask:"+current.getMask()
                    +",mcast:"+current.getMulticast()
                    +",bcast:"+current.getBroadcast()
                    +",ttl:"+current.getTTL());
        }
        
        if (current == null) {
            System.out.println("Strange, there is no subnet on this server!");
            System.exit(0);
        }
            
        // play with multicast setting on last in list
        
        // first test the "probe" value that is a special case
        int oldvalue = current.getMulticast();
        current.probeMulticast();
        current.update();
        System.out.println("Force multicast probe.");

        // then test the negation of oldvalue
        if (oldvalue > -1) {            // probe or enabled
            current.disableMulticast(); // change the Java instance
        } else {                        // disabled
            current.enableMulticast();  // change the Java instance
        }
        current.update();               // change on server
        System.out.println("Multicast setting has been changed.");

        RBSubnetRecs newrecs = rbServer.getSubnets("","");
        System.out.println(newrecs.size() + " subnets reloaded.");
        for (int i=0; i<newrecs.size(); i++) {
            RBSubnet sn = newrecs.elementAt(i);
            if (!sn.equals(current)) {
                continue;
            }
            System.out.println("Found modified subnet "+sn.getIPAddress()
                    +",mcast:"+sn.getMulticast());
            
            // restore multicast setting
            if (oldvalue < 0) {
                sn.disableMulticast();
            } else if (oldvalue == 0) {
                sn.probeMulticast();
            } else {
                sn.enableMulticast();
            }
            sn.update();
            
            System.out.println("Multicast setting has been restored.");
        }
        System.out.println("Done.");
    }

}
