/* TestWinPE2Update.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestWinPE2Update.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.RBWebExtensionStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

/*
 * Class to update an existing WinPE2 software package with drivers.
 */
public class TestWinPE2Update {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException, RBTimeoutException
    {
        if (args.length < 3) {
            System.out.println("usage: TestWinPE2Update <ip> <port> <password>\n");
            return;
        }

        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret, Level.ALL); 
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);
        
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        int hostIndex = -1;
        do {
            for (int i=0; i<availHosts.length; i++) {
                System.out.println(i + ") " + availHosts[i].getIPAddress()
                        + " MAC: " + availHosts[i].getMacAddress()
                        + " Description: " + availHosts[i].getDescription());
            }
            System.out.println("Please enter the number for the host running the WIE" +
            		"to use to perform the WinPE2 update: \n");
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.\n" +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        // Check to see if WAIK can be found on the target host
        System.out.println("\nTesting to see if WAIK is running on the target host..."); 
        String probeResult = rbServer.probeForWAIK(rbhost, 30);
        if (probeResult.equals("")) {
            System.out.println("Windows AIK could not be detected.  Please enter the\n" +
            		"path to WAIK on the target host.");
            probeResult = inReader.readLine().trim();
        } else {
            System.out.println("Probe result - found Windows AIK dir is: " + probeResult);
            probeResult = null;     // Will later pass this into the registerWinPE2UpdateActivity.
                                    // This is to show that if WAIK can be detected on the host,
                                    // there is no need to pass the waikDir into the 
                                    // registerWinPE2UpdateActivity method.
        }

        /*
         * Get the list of all WinPE2 software packages
         */
        RBSoftwarePackageRecs winPe2Recs = rbServer.getSoftwarePackages("[Content]='winpe2'", "");
        if (winPe2Recs.size() == 0) {
            System.out.println("This test needs a WinPE2 software package in order to run.\n"
                    + "There are none found. Exiting.");
            System.exit(1);
        }
        // We are assuming here there will only be a small number of these, so get them all at once.
        RBSoftwarePackage[] winPe2Pkgs = winPe2Recs.subList(0, winPe2Recs.size());
        int peIndex = -1;
        do {
            for (int i=0; i<winPe2Recs.size(); i++) {
                System.out.println(i + ") " + winPe2Pkgs[i].getDescription());
            }
            System.out.println("Enter the number of the WinPE2 software package you wish to update: \n");
            peIndex = Integer.parseInt(inReader.readLine().trim());
        } while (peIndex < 0 || peIndex >= winPe2Recs.size());
        RBSoftwarePackage myWinPe2 = winPe2Pkgs[peIndex];
        
        /*
         * Get list of all windows driver software packages
         */
        RBSoftwarePackageRecs allDriverRecs = rbServer.getSoftwarePackages("[Content]='win-drv'", "");
        if (allDriverRecs.size() == 0) {
            System.out.println("This test needs at least one Windows driver software package in order to run.\n"
                    + "There are none found. Exiting.");
            System.exit(1);
        }
        // We are assuming here there will only be a small number of these, so get them all at once.
        RBSoftwarePackage[] allDrivers = allDriverRecs.subList(0, allDriverRecs.size());
        ArrayList driversToAdd = new ArrayList();
        int drvIndex = -1;
        do {
            for (int i=0; i<allDriverRecs.size(); i++) {
           	 System.out.println(i + ") " + allDrivers[i].getDescription());
	        }
	        System.out.println("Enter the number of a driver you wish to add to the WinPE2 " +
	        		"image, or enter to continue.");
	        System.out.println("Currently selected drivers are: ");
	        Iterator it = driversToAdd.iterator();
	        while (it.hasNext()) {
	            RBSoftwarePackage drvr = (RBSoftwarePackage) it.next();
	            System.out.println(" - " + drvr.getDescription());
	        }
	        try {
	            drvIndex = Integer.parseInt(inReader.readLine().trim());
	            if (drvIndex >=0 && drvIndex < allDriverRecs.size()) {
	                driversToAdd.add(allDrivers[drvIndex]);
	            }
	        } catch (NumberFormatException e) {
	            break;
	        }
        } while (true);
        if (driversToAdd.size() == 0) {
            System.out.println("No drivers to add to the WinPE2 image.  Exiting.");
            System.exit(1);
        }
        
        
        Date now = new Date();
        Date stop = new Date(now.getTime()+1000*3600);
        Date expire = new Date(now.getTime()+1000*3600*24);
        RBActivity act = rbServer.registerWinPE2UpdateActivity("TestWinPE2 update activity", true, 
                null, rbhost, myWinPe2, (RBSoftwarePackage[]) driversToAdd.toArray(new RBSoftwarePackage[0]), 
                probeResult, now, stop, expire);
        
        
        
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent) {
            Thread.sleep(8000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events =  act.getEventsFromServer(lastTime);
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + 
                        " Progress Msg=" + e.getLastProgressMsg() + 
                        " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget ourActTarget = e.getRBActivityTarget();
                    System.out.println("Activity Target: Progress=" + ourActTarget.getProgress() + 
                            " Progress Msg=" + ourActTarget.getLastProgressMsg() + 
                            " Status=" + ourActTarget.getStatus());
                    
                    // Print out the specific results for the activity target
                    RBActivityTargetResults results = ourActTarget.getActivityTargetResults();
                    System.out.println("### Results for our WinPE2 update activity: ");
                    System.out.println("Status: " + results.getStatus());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
        
        System.out.println("\nTestWinPE2Update done.");
    }
    
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
